#include <stdio.h>
#include "pgp.h"
#include "usuals.h"  /* typedefs for byte, word16, boolean, etc. */
#include "hash.h"

int HashSize(int Hash)
{
    if (Hash == MD5_ALGORITHM_BYTE)
        return 16;
    else if (Hash == SHA1_ALGORITHM_BYTE)
        return 20;
    else if (Hash == RIPEMD160_ALGORITHM_BYTE)
        return 20;
    return 0;
}

void HashInit(struct HashContext *context, int Hash)
{
    context->Hash = Hash;
    context->hashsize = HashSize(context->Hash);
    if (context->Hash == MD5_ALGORITHM_BYTE)
        MD5Init(&context->ctx.md5ctx);
    else if (context->Hash == SHA1_ALGORITHM_BYTE)
        SHA1Init(&context->ctx.sha1ctx);
    else if (context->Hash == RIPEMD160_ALGORITHM_BYTE)
        RMDinit(&context->ctx.rmdctx);
}

void HashUpdate(struct HashContext *context, unsigned char const *buf, unsigned len)
{
    if (context->Hash == MD5_ALGORITHM_BYTE)
        MD5Update(&context->ctx.md5ctx, buf, len);
    else if (context->Hash == SHA1_ALGORITHM_BYTE)
        SHA1Update(&context->ctx.sha1ctx, buf, len);
    else if (context->Hash == RIPEMD160_ALGORITHM_BYTE)
        RMDUpdate(&context->ctx.rmdctx, buf, len);
}

void HashFinal(byte *buf, struct HashContext *context)
{
    if (context->Hash == MD5_ALGORITHM_BYTE)
        MD5Final(buf, &context->ctx.md5ctx);
    else if (context->Hash == SHA1_ALGORITHM_BYTE)
        SHA1Final(buf, &context->ctx.sha1ctx);
    else if (context->Hash == RIPEMD160_ALGORITHM_BYTE)
        RMDFinal(buf, &context->ctx.rmdctx);
}


int Hashfile0_len(struct HashContext *context, FILE *f, word32 longcount, int Hash)
{
    context->Hash = Hash;
    context->hashsize = HashSize(context->Hash);
    if (context->Hash == MD5_ALGORITHM_BYTE)
        return MDfile0_len(&context->ctx.md5ctx, f, longcount);
    else if (context->Hash == SHA1_ALGORITHM_BYTE)
        return SHA1file0_len(&context->ctx.sha1ctx, f, longcount);
    else if (context->Hash == RIPEMD160_ALGORITHM_BYTE)
        return RMDfile0_len(&context->ctx.rmdctx, f, longcount);
    return -1;
}

// Computes the message digest for a specified file
int Hashfile(struct HashContext *context, char *filename, int Hash)
{
    context->Hash = Hash;
    context->hashsize = HashSize(context->Hash);
    if (context->Hash == MD5_ALGORITHM_BYTE)
        return MDfile(&context->ctx.md5ctx, filename);
    else if (context->Hash == SHA1_ALGORITHM_BYTE)
        return SHA1file(&context->ctx.sha1ctx, filename);
    else if (context->Hash == RIPEMD160_ALGORITHM_BYTE)
        return RMDfile(&context->ctx.rmdctx, filename);
    return -1;
}

// Used to finish the message digest, including extra material.
void Hash_addbuffer (struct HashContext *context, byte *buf, int buflen, byte *digest)
{
    if (context->Hash == MD5_ALGORITHM_BYTE)
        MD_addbuffer(&context->ctx.md5ctx, buf, buflen, digest);
    else if (context->Hash == SHA1_ALGORITHM_BYTE)
        SHA1_addbuffer(&context->ctx.sha1ctx, buf, buflen, digest);
    else if (context->Hash == RIPEMD160_ALGORITHM_BYTE)
        RMD_addbuffer(&context->ctx.rmdctx, buf, buflen, digest);
}
